import 'dart:developer';
import 'package:award_app_api_service_flutter/network/uris/endpoints.dart';
import 'package:dio/dio.dart';
import 'exception/network_exception.dart';

class DioClient {
  Dio get _dio => Dio(BaseOptions(
        baseUrl: Endpoints.baseUrl,
        connectTimeout: Endpoints.connectionTimeout,
        receiveTimeout: Endpoints.receiveTimeout,
      ));

  // Headers:-----------------------------------------------------------------------

  Future<Map<String, dynamic>> authorizationHeaders(String token) async {
    final Map<String, dynamic> headers = {};
    try {
      if (token.length > 1) {
        headers.addAll({"authorization": "Bearer $token"});
      } else {}
    } catch (e) {
      log(e.toString());
    }
    return headers;
  }

  // ErrorHandler: -------------------------------------------------------------

  Future<NetworkException> _apiErrorHandler(DioError e) async {
    final response = e.response;
    final int statusCode = response?.statusCode ?? -1;
    final String statusMessage = response?.statusMessage ?? "エラーが発生しました";

    return NetworkException(
      message: statusMessage,
      statusCode: statusCode,
    );
  }

  // Get:-----------------------------------------------------------------------
  Future<dynamic> get(
    String uri, {
    Map<String, dynamic>? queryParameters,
    Future<Map<String, dynamic>>? headers,
    CancelToken? cancelToken,
    ProgressCallback? onReceiveProgress,
  }) async {
    try {
      final Response response = await _dio.get(
        uri,
        queryParameters: queryParameters,
        options: Options(
          contentType: Headers.jsonContentType,
          headers: await headers,
        ),
        cancelToken: cancelToken,
        onReceiveProgress: onReceiveProgress,
      );
      return response.data;
    } on DioError catch (e) {
      final exeception = await _apiErrorHandler(e);
      throw exeception;
    } catch (e) {
      rethrow;
    }
  }

  // Post:----------------------------------------------------------------------
  Future<dynamic> post(
    String uri, {
    data,
    Map<String, dynamic>? queryParameters,
    Future<Map<String, dynamic>>? headers,
    CancelToken? cancelToken,
    ProgressCallback? onSendProgress,
    ProgressCallback? onReceiveProgress,
  }) async {
    try {
      final Response response = await _dio.post(
        uri,
        data: data,
        queryParameters: queryParameters,
        options: Options(
          contentType: Headers.jsonContentType,
          headers: await headers,
        ),
        cancelToken: cancelToken,
        onSendProgress: onSendProgress,
        onReceiveProgress: onReceiveProgress,
      );
      return response.data;
    } on DioError catch (e) {
      final exeception = await _apiErrorHandler(e);
      throw exeception;
    }
  }

  // Delete:----------------------------------------------------------------------
  Future<dynamic> delete(
    String uri, {
    data,
    Map<String, dynamic>? queryParameters,
    Future<Map<String, dynamic>>? headers,
    CancelToken? cancelToken,
    ProgressCallback? onSendProgress,
    ProgressCallback? onReceiveProgress,
  }) async {
    try {
      final Response response = await _dio.delete(
        uri,
        data: data,
        queryParameters: queryParameters,
        options: Options(
          contentType: Headers.jsonContentType,
          headers: await headers,
        ),
        cancelToken: cancelToken,
      );
      return response.data;
    } on DioError catch (e) {
      final exeception = await _apiErrorHandler(e);
      throw exeception;
    }
  }

  // Put:----------------------------------------------------------------------
  Future<dynamic> put(
    String uri, {
    data,
    Map<String, dynamic>? queryParameters,
    Future<Map<String, dynamic>>? headers,
    CancelToken? cancelToken,
    ProgressCallback? onSendProgress,
    ProgressCallback? onReceiveProgress,
  }) async {
    try {
      final Response response = await _dio.put(
        uri,
        data: data,
        queryParameters: queryParameters,
        options: Options(
          contentType: Headers.jsonContentType,
          headers: await headers,
        ),
        cancelToken: cancelToken,
      );
      return response.data;
    } on DioError catch (e) {
      final exeception = await _apiErrorHandler(e);
      throw exeception;
    }
  }
}
